#include <vpi_user.h>

#include <cv.h>
#include <highgui.h>

/* global static used to store image data */
static IplImage *img_src = 0;

static int cv_open_calltf(char*c){
  vpiHandle sys_tf_ref, arg_iter;
  vpiHandle path_h, width_h, height_h, depth_h;
  s_vpi_value path_v, width_v, height_v, depth_v;

  /* Retrieve simulator system data */
  sys_tf_ref = vpi_handle(vpiSysTfCall,NULL);
  arg_iter = vpi_iterate(vpiArgument, sys_tf_ref);

  /* Scan args */
  path_h = vpi_scan(arg_iter);
  width_h = vpi_scan(arg_iter);
  height_h = vpi_scan(arg_iter);
  depth_h = vpi_scan(arg_iter);
  
  /* Retrieve first string arg */
  path_v.format = vpiStringVal;
  vpi_get_value(path_h, &path_v);

  /* do real work */
  {
    IplImage *img = cvLoadImage(path_v.value.str,1);
    if(img == NULL) {
      vpi_printf("Error opening file %s\n",path_v.value.str);
      return -1;
    }
    if(img->nChannels != 1) {
      img_src = cvCreateImage(cvSize(img->width,img->height), IPL_DEPTH_8U, 1);
      cvCvtColor(img, img_src, CV_BGR2GRAY);
      cvReleaseImage(&img);
    } else {
      img_src = img;
    }
  }
  
  vpi_printf("Opening file %s: %d x %d\n",path_v.value.str,img_src->width,img_src->height);

  /* set ouput vars */
  width_v.format = vpiIntVal;
  width_v.value.integer = img_src->width;
  vpi_put_value(width_h, &width_v, 0, vpiNoDelay);

  height_v.format = vpiIntVal;
  height_v.value.integer = img_src->height;
  vpi_put_value(height_h, &height_v, 0, vpiNoDelay);
  
  depth_v.format = vpiIntVal;
  depth_v.value.integer = img_src->nChannels;
  vpi_put_value(depth_h, &depth_v, 0, vpiNoDelay);

  return 0;
}

static int cv_pixel_calltf(char*c){
  vpiHandle sys_tf_ref, arg_iter;
  vpiHandle row_h, col_h, dout_h;
  s_vpi_value value;

  unsigned int row, col;
  
  sys_tf_ref = vpi_handle(vpiSysTfCall,NULL);
  arg_iter = vpi_iterate(vpiArgument, sys_tf_ref);

  row_h = vpi_scan(arg_iter);
  col_h = vpi_scan(arg_iter);
  dout_h = vpi_scan(arg_iter);

  //
  value.format = vpiIntVal;
  vpi_get_value(row_h, &value);
  row = value.value.integer;

  vpi_get_value(col_h, &value);
  col = value.value.integer;

  /* do it */
  value.value.integer = (unsigned char)img_src->imageData[ (row*img_src->widthStep) + col ];
  
  /* return it */
  vpi_put_value(dout_h, &value, 0, vpiNoDelay);
  
  return 0;
}

static int cv_close_calltf(char*c){
  vpi_printf("cv_close\n");

  return 0;
}

void sensor_register(void)
{
  s_vpi_systf_data tf_data;
  
  tf_data.type      = vpiSysTask;
  tf_data.tfname    = "$cv_open";
  tf_data.calltf    = cv_open_calltf;
  tf_data.compiletf = 0;
  tf_data.sizetf    = 0;
  tf_data.user_data = "$cv_open";
  vpi_register_systf(&tf_data);
  
  tf_data.type      = vpiSysTask;
  tf_data.tfname    = "$cv_pixel";
  tf_data.calltf    = cv_pixel_calltf;
  tf_data.compiletf = 0;
  tf_data.sizetf    = 0;
  tf_data.user_data = "$cv_pixel";
  vpi_register_systf(&tf_data);
  
  tf_data.type      = vpiSysTask;
  tf_data.tfname    = "$cv_close";
  tf_data.calltf    = cv_close_calltf;
  tf_data.compiletf = 0;
  tf_data.sizetf    = 0;
  tf_data.user_data = "$cv_close";
  vpi_register_systf(&tf_data);

}


void (*vlog_startup_routines[])() = {
  sensor_register,
  0
};


