`timescale 1ns/1ps

module	sensor (MCLK,	// Master clock
		MRST,	// Master Reset - active low
		ARO,	// Array read Out.
		ARST,	// Array Reset. Active low
		OE,	// output enable active low
		SCL,	// I2C clock
		SDA,	// I2C data
		OFST,	// I2C address ofset
		D,	// data output
		DCLK,	// Data output clock
		BPF,	// Black Pixel Flag
		HACT,	// Horizontal Active
		VACT);  // Vertical Active

   input			MCLK,	// Master clock
				MRST,	// Master Reset - active low
				ARO,	// Array read Out.
				ARST,	// Array Reset. Active low
				OE,		// output enable active low
				SCL;	// I2C clock
   inout			SDA;	// I2C data
   input			OFST;	// I2C address ofset

   //output reg [9:0] 		D;	// data output
   output reg [7:0] 		D;	// data output
   output			DCLK,	// Data output clock
				BPF,	// Black Pixel Flag
				HACT,	// Horizontal Active
				VACT;	// Vertical Active
   
   parameter 			source = 0 ; 			
   parameter 			nrowb   =	1;	// number of "blank rows" from vact to 1-st hact
   parameter 			nrowa	=	1;	// number of "blank rows" from last hact to end of vact
   parameter 			nAV	=	24;	// clocks from ARO to VACT (actually from en_dclkd)
   parameter 			nbpf	=	20;	// bpf length
   parameter 			ngp1	=	8;	// bpf to hact
   parameter 			nVLO	=	1;	// VACT=0 in video mode (clocks)
   parameter 			tMD	=	4;	
   parameter 			tDDO	=	2;	


   parameter 			s_stop=		0;
   parameter 			s_preVACT=	1;
   parameter 			s_firstline=2;
   parameter 			s_BPF=		3;
   parameter 			s_preHACT=	4;
   parameter 			s_HACT=		5;
   parameter 			s_afterHACT=6;
   parameter 			s_lastline=	7;
   parameter 			s_frame_done=8;

   reg [15:0] 			ncols, nrows;
   
   reg [15:0] 			t_preVACT =	nAV;		
   
   reg [15:0] 			t_BPF =		nbpf;
   reg [15:0] 			t_preHACT =	ngp1;
   
   wire [15:0] 			t_firstline;
   wire [15:0] 			t_lastline;
   
   reg 				c;		// internal data out clock
   reg				stopped;
   wire #1 			stoppedd=stopped;
   reg				ibpf, ihact, ivact;
   reg				arst1;	//
   reg [10:0] 			col;	// current row
   reg [10:0] 			row;	// current column;
   reg [3:0] 			state;
   reg [15:0] 			cntr;
   wire [10:0] 			cold;
   wire [10:0] 			rowd;
   wire [3:0] 			stated;
   wire [15:0] 			cntrd;
   wire 			NMRST=!MRST;

   wire [10:0] 			row_index=row[10:0];
   wire [10:0] 			col_index=col[10:0];

   // random
   reg [4:0] 			channels;
   wire [15:0] 			t_afterHACT;

   assign t_firstline = nrowb * ncols + 1;
   assign t_lastline  =	nrowa * ncols + 1;
   
   assign #1 			cold=	col;
   assign #1 			rowd=	row;
   assign #1 			stated=	state;
   assign #1 			cntrd=	cntr;

   assign #tDDO 		BPF	= ibpf;
   assign #tDDO 		HACT= ihact;
   assign #tDDO 		VACT= ivact;
   assign			DCLK= c;

   assign 			t_afterHACT = 352;			
   
   // Setup parameters
   initial begin
      $cv_open(source, ncols, nrows, channels);
      
      $display ("sensor parameters");
      $display ("    -- ncols = %d (numer of clocks in HACT)",ncols);
      $display ("    -- nrows = %d (number of rows)",nrows);

      c=0;
      //	{ibpf,ihact,ivact}=0;
      stopped = 1;
      arst1 = 0;
      row = 0;
      col = 0;
   end

   reg [9:0] cv_din;
   
   initial forever begin
      #tDDO if( !OE && (ihact || ibpf) ) begin
	 $cv_pixel(row_index, col_index, cv_din);
	 D = cv_din;
      end
      else
	D <= {10{1'bz}};
   end

   always @ (NMRST) begin
      c=0;
      //	{ibpf,ihact,ivact}=0;
      stopped=1;
      arst1=0;
      //	row=0;
      //	col=0;
   end

   always begin
      @ (posedge MCLK) begin
	 #tMD	c = !stoppedd;
      end
      @ (negedge MCLK) begin
	 #tMD	c = 1'b0;
      end
   end

   always @ (posedge MCLK) begin
      #1	stopped= !arst1 || (stoppedd && !ARO) ;
      #1	arst1=ARST;
   end

   always @ (posedge stoppedd or posedge c) begin
      if (stoppedd) begin
	 {ibpf,ihact,ivact}=0;
	 row=0;
	 col=0;
	 //		id=0;
	 state=0;
	 cntr=0;
      end else if (|cntrd != 0) begin
	 #1 cntr=cntrd-1;
	 if (BPF || HACT) col=cold+1;
      end else begin
	 case (stated)
	   s_stop: begin
	      cntr=	t_preVACT-1;
	      state=	s_preVACT;
	   end
	   s_preVACT: begin
	      ivact=	1'b1;
	      cntr=	t_firstline-1;
	      state=	s_firstline;
	   end
 	   s_firstline: begin
	      ibpf=	1'b1;
	      col=	0;
	      row=	0;
	      cntr=	t_BPF-1;
	      state=	s_BPF;
	   end
	   s_BPF: begin
	      ibpf=	1'b0;
	      cntr=	t_preHACT-1;
	      state=	s_preHACT;
	   end
	   s_preHACT: begin
	      ihact=	1'b1;
	      col=	0;
	      cntr=     ncols-1;
	      state=	s_HACT;
	   end
	   s_HACT: begin
	      ihact=	1'b0;
	      row=	rowd+1;
	      cntr=	t_afterHACT-1;
	      state=	s_afterHACT;
	   end
	   s_afterHACT:
	     if (rowd == nrows) begin
		cntr=	t_lastline-1;
		state=	s_lastline;
	     end else begin
		ibpf=	1'b1;
		col=	0;
		cntr=	t_BPF-1;
		state=	s_BPF;
	     end
	   s_lastline: begin
	      ivact=	1'b0;
	      state=	s_frame_done;
	      cntr=nVLO;
	   end
	   s_frame_done: if (OFST) begin
	      ivact=	1'b1;
	      cntr=	t_firstline-1;
	      state=	s_firstline;
	   end
	 endcase

      end
   end



endmodule
